@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2007-2012 Sourcefire, Inc.
+ *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -38,10 +39,6 @@
 #include <sys/resource.h>
 #endif
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "others.h"
 #include "global.h"
 #include "manager.h"
@@ -71,8 +68,8 @@ int main(int argc, char **argv)
 	struct optstruct *opts;
 	const struct optstruct *opt;
 
-    if(check_flevel())
-	exit(2);
+    if (check_flevel())
+	    exit(2);
 
 #if !defined(_WIN32) && !defined(C_BEOS)
     sigemptyset(&sigset);
@@ -80,9 +77,7 @@ int main(int argc, char **argv)
     sigprocmask(SIG_SETMASK, &sigset, NULL);
 #endif
 
-#if defined(_WIN32)
     cl_initialize_crypto();
-#endif
 
 
     if((opts = optparse(NULL, argc, argv, 1, OPT_CLAMSCAN, 0, NULL)) == NULL) {
@@ -195,20 +190,23 @@ int main(int argc, char **argv)
 
     optfree(opts);
 
+    cl_cleanup_crypto();
+
     return ret;
 }
 
 void help(void)
 {
-
     mprintf_stdout = 1;
 
     mprintf("\n");
-    mprintf("                       Clam AntiVirus Scanner %s\n", get_version());
-    printf("           By The ClamAV Team: http://www.clamav.net/team\n");
-    printf("           (C) 2007-2009 Sourcefire, Inc.\n\n");
-
-    mprintf("    --help                -h             Print this help screen\n");
+    mprintf("                       Clam AntiVirus: Scanner %s\n", get_version());
+    printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
+    printf("           (C) 2019 Cisco Systems, Inc.\n");
+    mprintf("\n");
+    mprintf("    clamscan [options] [file/directory/-]\n");
+    mprintf("\n");
+    mprintf("    --help                -h             Show this help\n");
     mprintf("    --version             -V             Print version number\n");
     mprintf("    --verbose             -v             Be verbose\n");
     mprintf("    --archive-verbose     -a             Show filenames inside scanned archives\n");
@@ -222,8 +220,9 @@ void help(void)
     mprintf("\n");
     mprintf("    --tempdir=DIRECTORY                  Create temporary files in DIRECTORY\n");
     mprintf("    --leave-temps[=yes/no(*)]            Do not remove temporary files\n");
-    mprintf("    --database=FILE/DIR   -d FILE/DIR    Load virus database from FILE or load\n");
-    mprintf("                                         all supported db files from DIR\n");
+    mprintf("    --gen-json[=yes/no(*)]               Generate JSON description of scanned file(s). JSON will be printed and also-\n");
+    mprintf("                                         dropped to the temp directory if --leave-temps is enabled.\n");
+    mprintf("    --database=FILE/DIR   -d FILE/DIR    Load virus database from FILE or load all supported db files from DIR\n");
     mprintf("    --official-db-only[=yes/no(*)]       Only load official signatures\n");
     mprintf("    --log=FILE            -l FILE        Save scan report to FILE\n");
     mprintf("    --recursive[=yes/no(*)]  -r          Scan subdirectories recursively\n");
@@ -243,7 +242,7 @@ void help(void)
     mprintf("    --bytecode[=yes(*)/no]               Load bytecode from the database\n");
     mprintf("    --bytecode-unsigned[=yes/no(*)]      Load unsigned bytecode\n");
     mprintf("    --bytecode-timeout=N                 Set bytecode timeout (in milliseconds)\n");
-    mprintf("    --bytecode-statistics[=yes/no(*)]    Collect and print bytecode statistics\n");
+    mprintf("    --statistics[=none(*)/bytecode/pcre] Collect and print execution statistics\n");
     mprintf("    --detect-pua[=yes/no(*)]             Detect Possibly Unwanted Applications\n");
     mprintf("    --exclude-pua=CAT                    Skip PUA sigs of category CAT\n");
     mprintf("    --include-pua=CAT                    Load PUA sigs of category CAT\n");
@@ -252,22 +251,29 @@ void help(void)
     mprintf("    --structured-ssn-count=N             Min SSN count to generate a detect\n");
     mprintf("    --structured-cc-count=N              Min CC count to generate a detect\n");
     mprintf("    --scan-mail[=yes(*)/no]              Scan mail files\n");
-    mprintf("    --phishing-sigs[=yes(*)/no]          Signature-based phishing detection\n");
-    mprintf("    --phishing-scan-urls[=yes(*)/no]     URL-based phishing detection\n");
+    mprintf("    --phishing-sigs[=yes(*)/no]          Enable email signature-based phishing detection\n");
+    mprintf("    --phishing-scan-urls[=yes(*)/no]     Enable URL signature-based phishing detection\n");
+    mprintf("    --heuristic-alerts[=yes(*)/no]       Heuristic alerts\n");
     mprintf("    --heuristic-scan-precedence[=yes/no(*)] Stop scanning as soon as a heuristic match is found\n");
-    mprintf("    --phishing-ssl[=yes/no(*)]           Always block SSL mismatches in URLs (phishing module)\n");
-    mprintf("    --phishing-cloak[=yes/no(*)]         Always block cloaked URLs (phishing module)\n");
-    mprintf("    --partition-intersection[=yes/no(*)] Detect partition intersections in raw disk images using heuristics.\n");
-    mprintf("    --algorithmic-detection[=yes(*)/no]  Algorithmic detection\n");
+    mprintf("    --normalize[=yes(*)/no]              Normalize html, script, and text files. Use normalize=no for yara compatibility\n");
     mprintf("    --scan-pe[=yes(*)/no]                Scan PE files\n");
     mprintf("    --scan-elf[=yes(*)/no]               Scan ELF files\n");
     mprintf("    --scan-ole2[=yes(*)/no]              Scan OLE2 containers\n");
     mprintf("    --scan-pdf[=yes(*)/no]               Scan PDF files\n");
     mprintf("    --scan-swf[=yes(*)/no]               Scan SWF files\n");
     mprintf("    --scan-html[=yes(*)/no]              Scan HTML files\n");
+    mprintf("    --scan-xmldocs[=yes(*)/no]           Scan xml-based document files\n");
+    mprintf("    --scan-hwp3[=yes(*)/no]              Scan HWP3 files\n");
     mprintf("    --scan-archive[=yes(*)/no]           Scan archive files (supported by libclamav)\n");
-    mprintf("    --detect-broken[=yes/no(*)]          Try to detect broken executable files\n");
-    mprintf("    --block-encrypted[=yes/no(*)]        Block encrypted archives\n");
+    mprintf("    --alert-broken[=yes/no(*)]           Alert on broken executable files (PE & ELF)\n");
+    mprintf("    --alert-encrypted[=yes/no(*)]        Alert on encrypted archives and documents\n");
+    mprintf("    --alert-encrypted-archive[=yes/no(*)] Alert on encrypted archives\n");
+    mprintf("    --alert-encrypted-doc[=yes/no(*)]    Alert on encrypted documents\n");
+    mprintf("    --alert-macros[=yes/no(*)]           Alert on OLE2 files containing VBA macros\n");
+    mprintf("    --alert-exceeds-max[=yes/no(*)]      Alert on files that exceed max file size, max scan size, or max recursion limit\n");
+    mprintf("    --alert-phishing-ssl[=yes/no(*)]     Alert on emails containing SSL mismatches in URLs\n");
+    mprintf("    --alert-phishing-cloak[=yes/no(*)]   Alert on emails containing cloaked URLs\n");
+    mprintf("    --alert-partition-intersection[=yes/no(*)] Alert on raw DMG image files containing partition intersections\n");
     mprintf("    --nocerts                            Disable authenticode certificate chain verification in PE files\n");
     mprintf("    --dumpcerts                          Dump authenticode certificate chain in PE files\n");
     mprintf("\n");
@@ -283,10 +289,15 @@ void help(void)
     mprintf("    --max-ziptypercg=#n                  Maximum size zip to type reanalyze\n");
     mprintf("    --max-partitions=#n                  Maximum number of partitions in disk image to be scanned\n");
     mprintf("    --max-iconspe=#n                     Maximum number of icons in PE file to be scanned\n");
-    mprintf("    --enable-stats                       Enable statistical reporting of malware\n");
-    mprintf("    --disable-pe-stats                   Disable submission of individual PE sections in stats submissions\n");
-    mprintf("    --stats-timeout=#n                   Number of seconds to wait for waiting a response back from the stats server\n");
-    mprintf("    --stats-host-id=UUID                 Set the Host ID used when submitting statistical info.\n");
+    mprintf("    --max-rechwp3=#n                     Maximum recursive calls to HWP3 parsing function\n");
+#if HAVE_PCRE
+    mprintf("    --pcre-match-limit=#n                Maximum calls to the PCRE match function.\n");
+    mprintf("    --pcre-recmatch-limit=#n             Maximum recursive calls to the PCRE match function.\n");
+    mprintf("    --pcre-max-filesize=#n               Maximum size file to perform PCRE subsig matching.\n");
+#endif /* HAVE_PCRE */
+    mprintf("    --disable-cache                      Disable caching and cache checks for hash sums of scanned files.\n");
+    mprintf("\n");
+    mprintf("Pass in - as the filename for stdin.\n");
     mprintf("\n");
     mprintf("(*) Default scan settings\n");
     mprintf("(**) Certain files (e.g. documents, archives, etc.) may in turn contain other\n");
