@@ -135,6 +135,8 @@ my %CONF = (
     'HAVE_STRING_H' => '1',
     'HAVE_STRLCAT' => -1,
     'HAVE_STRLCPY' => -1,
+    'HAVE_STRNDUP' => -1,
+    'HAVE_STRNSTR' => -1,
     'HAVE_SYSCONF_SC_PAGESIZE' => -1,
     'HAVE_SYSTEM_TOMMATH' => -1,
     'HAVE_SYS_DL_H' => -1,
